@@ -5,10 +5,11 @@ use warnings;
 
 use parent qw{ HTML::Parser };
 
-our $VERSION = '0.200';
+our $VERSION = '0.201';
 
 our $DEBUG = 0;
 
+my @stacked = qw{ current_table current_row current_element };
 
 sub start
 {
@@ -20,13 +21,16 @@ sub start
 # Store the incoming details in the current 'object'.
 	if ($tag eq 'table') {
 		my $table = $attr;
+		push @{ $self->{STORE}{stack} }, {
+			map { $_ => $self->{STORE}{$_} } @stacked };
 		push @{$self->{STORE}->{tables}}, $table;
 		$self->{STORE}->{current_table} = $table;
+		$self->{STORE}->{current_row} = undef;
+		$self->{STORE}->{current_element} = undef;
 
 	} elsif ($tag eq 'th') {
 		my $th = $attr;
 		push @{$self->{STORE}->{current_table}->{headers}}, $th;
-		$self->{STORE}->{current_header} = $th;
 		$self->{STORE}->{current_element} = $th;
 
 	} elsif ($tag eq 'tr') {
@@ -38,7 +42,6 @@ sub start
 	} elsif ($tag eq 'td') {
 		my $td = $attr;
 		push @{$self->{STORE}->{current_row}->{cells}}, $td;
-		$self->{STORE}->{current_data_cell} = $td;
 		$self->{STORE}->{current_element} = $td;
 
 	} elsif ($tag eq 'caption') {
@@ -87,27 +90,18 @@ sub end
 
 # Turn off the current object
 	if ($tag eq 'table') {
-		$self->{STORE}->{current_table} = undef;
-		$self->{STORE}->{current_row} = undef;
-		$self->{STORE}->{current_data_cell} = undef;
-		$self->{STORE}->{current_header} = undef;
-		$self->{STORE}->{current_element} = undef;
+		my $prev = pop @{ $self->{STORE}{stack} } || [];
+		$self->{STORE}{$_} = $prev->{$_} for @stacked;
 
 	} elsif ($tag eq 'th') {
 		$self->{STORE}->{current_row} = undef;
-		$self->{STORE}->{current_data_cell} = undef;
-		$self->{STORE}->{current_header} = undef;
 		$self->{STORE}->{current_element} = undef;
 
 	} elsif ($tag eq 'tr') {
 		$self->{STORE}->{current_row} = undef;
-		$self->{STORE}->{current_data_cell} = undef;
-		$self->{STORE}->{current_header} = undef;
 		$self->{STORE}->{current_element} = undef;
 
 	} elsif ($tag eq 'td') {
-		$self->{STORE}->{current_data_cell} = undef;
-		$self->{STORE}->{current_header} = undef;
 		$self->{STORE}->{current_element} = undef;
 
 	} elsif ($tag eq 'caption') {
@@ -134,16 +128,21 @@ sub parse
 {
 	my ($self, $data) = @_;
 
-	$self->{STORE} = undef;
+	unless ( defined $data ) {	# RT 7262
+	    require Carp;
+	    Carp::croak( 'Argument must be defined' );
+	}
 
-# Ensure the following keys exist
-	$self->{STORE}->{current_data_cell} = undef;
-	$self->{STORE}->{current_row} = undef;
-	$self->{STORE}->{current_table} = undef;
+	$self->{STORE} = {
+	    stack	=> [],
+	};
 
 	$self->SUPER::parse($data);
 
-	return $self->{STORE}->{tables};
+	my $tables = $self->{STORE}{tables};
+	delete $self->{STORE};
+
+	return $tables;
 }
 
 
@@ -180,6 +179,12 @@ Each time a table is encountered, data will be stored in an array consisting
 of a hash of whatever was discovered about the table -- id, name, border,
 cell spacing etc, and of course data contained within the table.
 
+Tables appear in the output in the order in which they are encountered.
+If a table is nested inside a cell of another table, it will appear
+after the containing table in the output, and any connection between the
+two will be lost. As of version 0.200_01, the appearance of a nested
+table should not cause any truncation of the containing table.
+
 The format of each hash will look something like
 
   attributes            keys from the attributes of the <table> tag
@@ -243,6 +248,9 @@ Called with the HTML to parse. This is all the application needs to do.
 The return value will be an arrayref containing each table encountered, in the
 format detailed above.
 
+This method will C<croak()> if the argument is not defined, or not
+specified.
+
 =item DEBUG
 
 Not a method, but a class variable. Set to 1 to cause debugging output
@@ -263,7 +271,7 @@ Nothing.
 This module is a very specific tool to address a very specific problem.
 One of the following modules may better address your needs.
 
-L<HTML::Parser|HTML::Parser>. This is a general HTML parser, which form
+L<HTML::Parser|HTML::Parser>. This is a general HTML parser, which forms
 the basis for this module.
 
 L<HTML::TreeBuilder|HTML::TreeBuilder>. This is a general HTML parser,
@@ -283,7 +291,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 Copyright (C) 2002 Simon Drabble
 
-Copyright (C) 2017 Thomas R. Wyant, III
+Copyright (C) 2017-2018 Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
